#
# WEBMIRROR
#
#
# Written by Renaud Deraison <deraison@nessus.org>
#    Viciously mangled by H D Moore <hdmoore@digitaldefense.net>
#
# This plugin mirrors the paths used by a website. We typically care
# to obtain the list of CGIs installed on the remote host, as well as
# the path they are installed under. 
#
# Note that this plugin does not properly check for the syntax of the
# HTML pages returned : it tries to extract as much info as it
# can. We don't care about the pages extensions either (but we do
# case about the mime types)
#
# This plugin takes a really long time to complete, so it updates
# the KB as soon as data is found (as it's likely to be killed
# by nessusd against huge sites)
#
#
# Credit goes to webmirror.pl by t-0micr0n - mostly for the idea of 
# saving time and code complexity by not checking if the HTML syntax is 
# correctly respected
#
#
#
# This plugin only works properly with the libnasl 1.1.x and newer
#
# See the Nessus Scripts License for details
#

if(description)
{
 script_id(10662);
 script_version("$Revision: 1.41 $");
 
 name["english"] = "Web mirroring";
 name["francais"] = "Web mirroring";
 script_name(english:name["english"], francais:name["francais"]);
 
 desc["english"] = "
This script makes a mirror of the remote web site(s)
and extracts the list of CGIs that are used by the remote
host.

It is suggested you give a high timeout value to
this plugin and that you change the number of
pages to mirror in the 'Options' section of
the client.

Risk factor : None";

 script_description(english:desc["english"]);
 
 summary["english"] = "Performs a quick web mirror";
 summary["francais"] = "Effectue un miroir rapide de site web";
 
 script_summary(english:summary["english"], francais:summary["francais"]);
 
 script_category(ACT_GATHER_INFO);
 
 
 script_copyright(english:"This script is Copyright (C) 2001 Renaud Deraison",
		francais:"Ce script est Copyright (C) 2001 Renaud Deraison");
 family["english"] = "CGI abuses";
 family["francais"] = "Abus de CGI";
 script_family(english:family["english"], francais:family["francais"]);
 script_dependencie("find_service.nes", "httpver.nasl", "DDI_Directory_Scanner.nasl");
 script_require_ports("Services/www", 80);
 script_add_preference(name:"Number of pages to mirror : ",
 			type:"entry",
			value:"25");
 script_add_preference(name:"Start page : ",
 			type:"entry",
			value:"/");			
 exit(0);
}

include("http_func.inc");
	     
#------------------------------------------------------#
# Globals                                              #
#------------------------------------------------------#

exit(0); # Currently broken

hostname = get_host_name();
start_page = script_get_preference("Start page : ");
if(isnull(start_page) || start_page == "")start_page = "/";

port = get_kb_item("Services/www");
if(!port) port = 80;

misc_report = string("\nMiscellaneous:\n\n");
misc_count = 0;

#
# Is there a service running ?
#
if(!get_port_state(port))exit(0);
soc = http_open_socket(port);
if(!soc)exit(0);


max_pages = script_get_preference("Number of pages to mirror : ");
if(isnull(max_pages) || max_pages == 0)max_pages = 100;

quote = raw_string(0x22);
close_parenth = raw_string(0x29);

debug = 0;

urls[0] = start_page;
num_urls = 1;

global_var url_hash;

cgis[0] = "";
num_cgis = 0;


args[0] = "";
num_args = 0;

url200[0] = "";
num_200 = 0;

url404[0] = "";
num_404 = 0;

cur_args = "";
cur_cgi = 0;

___saveme = 0;
level = 0;
num_extensions = 1;
extensions[0] = "html";
pages["html"] = 0;


#------------------------------------------------------#
# Directory management stuff                           #
#------------------------------------------------------#

dirs[0] = "/";
num_dirs = 0;


function check_dir_list (dir)
{
    for (CDL=0; dirs[CDL]; CDL=CDL+1)
    {
        if (dirs[CDL] == dir)
        {
            return(1);
        }
    }
    return(0);
}

function add_dir_list (dir, setkb)
{
    if (check_dir_list(dir:dir) == 0)
    {  
        dirs[num_dirs] = dir; 
        
        if (setkb)
        {
            dir_key = string("www/", port, "/content/directories/", num_dirs);
            if(debug) display("Setting KB key: ", dir_key, " to '", dir, "'\n");
            set_kb_item(name:dir_key, value:dir);
        }
        
        num_dirs = num_dirs + 1;  
    }
}

#------------------------------------------------------#
# Visited URL tracking                                 #
#------------------------------------------------------#

vurls[0] = 0;
num_vurls = 0;

function check_vurl_list (vurl)
{
    for (VUL=0; vurls[VUL]; VUL=VUL+1)
    {
        if (vurls[VUL] == vurl)
        {
            return(1);
        }
    }
    return(0);
}

function add_vurl_list (vurl)
{
    if (check_vurl_list(vurl:vurl) == 0)
    {  
        vurls[num_vurls] = vurl;
        num_vurls = num_vurls + 1;
    }
}

#------------------------------------------------------#
# Miscellaneous Report Tracking                        #
#------------------------------------------------------#

misc_reps[0] = 0;
num_misc_reps = 0;

function check_misc_rep_list (rep)
{
    for (VUL=0; misc_reps[VUL]; VUL=VUL+1)
    {
        if (misc_reps[VUL] == rep)
        {
            return(1);
        }
    }
    return(0);
}

function add_misc_rep_list (rep)
{
    if (check_misc_rep_list(rep:rep) == 0)
    {  
        misc_reps[num_vurls] = rep;
        num_misc_reps = num_misc_reps + 1;
        return(1);
    }
    return(0);
}
#------------------------------------------------------#
# Keep track of valid & invalid urls                   #
#------------------------------------------------------#
function add_200(url)
{ 
  url200[num_200] = url;
  num_200 = num_200 + 1;
  
  add_dir(dir_url:url);
  
  #
  # We put the valid URLs in the KB, so that they can be re-used
  # by other plugins.
  #
  if(debug > 2)display(string("www/", port, "/content/pages/", num_200), "=" ,url, "\n");
  set_kb_item(name:string("www/", port, "/content/pages/", num_200), value:url);
  
  ___url = ereg_replace(string:url,
  			pattern:"(.*)\?.*",
			replace:"\1");
  if("." >< ___url)
  {
  if(! match(pattern:"*/", string:___url))
  {
  xt = ereg_replace(pattern:".*\.([^\.]*)$",
  		     string:___url,
		     replace:"\1");
   if(!("/" >< xt))
    {
     num = pages[xt];
     pages[xt] = num + 1;
     set_kb_item(name:string("www/", port, "/content/extensions/", xt, "/",
     		pages[xt]), value:url);
     }
    }
   }	     
}

function add_404(url)
{
  url404[num_404] = url;
  num_404 = num_404 + 1;
  
  add_dir(dir_url:url);
  # set_kb_item ?
}

#------------------------------------------------------#
# Add a CGI to our array of CGIs to visit              #
#------------------------------------------------------#
function add_cgi(cgi)
{
  if ("--WEBBOT-SELF--" >< cgi)
  {
    if(debug)display("**** add_cgi -> ignoring bogus frontpage cgi\n");
    return(num_cgis);
  }
  
  #
  # Replace http://blah/tag by /tag
  # Skip the url if it's not the same host
  #
  if(match(pattern:"*http*", 
	  string:cgi,
	  icase:1))
  {
    hn = ereg_replace(pattern:".*http://([^/]*)(/|$).*",
		     string:cgi,
		     icase:1,
		     replace:"\1");

    myname =  hostname;
    if(myname == hn)
    {
    cgi = ereg_replace(pattern:".*http://[^/]*(/.*)",
		     string:cgi,
		     icase:1,
		     replace:"\1");
    }
    else {
        if(debug)display("FAILED! ", hn, " != ", myname);
	return(0);
    }
  }
  
  
  for(_i = 0 ; _i < num_cgis ; _i = _i + 1)
  {
    if(cgi == cgis[_i])
      {
      #display("return ", cgi, "\n");
      return(_i);
      }
  }

  if(debug)display("add cgi ", cgi, "\n");
  add_dir(dir_url:cgi);
  
  cgis[num_cgis] = cgi;
  num_cgis = num_cgis + 1;
  
  #
  # Put the CGI in the KB, so that it may be re-used by another
  # plugin.
  #
  set_kb_item(name:string("www/", port, "/content/cgi/", num_cgis), value:cgi);	
  if(debug > 2)display("**** add_cgi returning ", num_cgis, "\n");
  return(num_cgis);
}

#------------------------------------------------------#
# Extracts a list of arguments from a 'cmd-line' arg   #
# (that is, /foo.php?id=a&name=b -> "id name"          #
#------------------------------------------------------#
function parse_cmdline_args(cmd)
{
 _ret = "";
 while(cmd)
 {
  _cmd = ereg_replace(string:cmd,
  		      pattern:"([^&]*)&.*",
		      replace:"\1");
  if(!_cmd)
  {
   cmd = 0;
  }
  else
  {		      
   _arg = ereg_replace(string:_cmd,
   			pattern:"([^=]*)=.*",
			replace:"\1");
			
   _val = ereg_replace(string:_cmd,
   			pattern:"([^=]*)=([^&]*).*",
			replace:"\2");
   if((_val == _cmd))_val = "";
   	
   if(!(string(_arg, " ") >< _ret))
   {
   _ret = string(_ret, _arg, " ");
   if(_val)
    _ret = string(_ret, "[", _val, "] ");
   }
  cmd = cmd - _cmd;	     
  } 
 }
 if(strlen(_ret) > 1)
 {
 return(_ret);
 }
 else return(0);
}


#------------------------------------------------------#
# Add an URL in our array of URLs to visit             #
#------------------------------------------------------#
function add_url(url)
{

  if(debug > 2)display("**** add_url(): ", url, "\n");
  
  # strip out the apache multi-view links
  url = ereg_replace(pattern:"\?[NSMD]=[AD]$", string:url, replace:"");

  #
  # Replace http://blah/tag by /tag
  # Skip the url if it's not the same host
  #
  if(match(pattern:"http*", 
	  string:url,
	  icase:1))
  {
    hn = ereg_replace(pattern:"^http://([^/]*)/.*$",
		     string:url,
		     icase:1,
		     replace:"\1");
    if(hn == url)return(0);		     
    rest = ereg_replace(pattern:"^http://[^/]*(/.*)$",
		     string:url,
		     icase:1,
		     replace:"\1");		     

    myname = get_host_name();
    if(!rest)
     return(0);
    if(myname == hn)
    {
      url = rest;
    }
    else {
	return(0);
    }
  }

#while ("//" >< url)
#{
#  display("Too many / in ", url, "\n");
#  url = ereg_replace(pattern:"^(.*)///*(.*)$", string: url, replace: "\1/\2");
#}
  
  if("?" >< url)
	  {
	   _cgi = ereg_replace(pattern:"([^?]*)\?.*",
	   		       replace:"\1",
			       string:url);
	   _args = ereg_replace(pattern:"[^?]*\?(.*)",
	   			replace:"\1",
				string:url);
	   _args = parse_cmdline_args(cmd:_args);
	   if(debug)display("call to add_cgi()\n");		     
	   xyz = add_cgi(cgi:_cgi);
	   if(xyz) args[xyz - 1] = _args;
	  }
# for(_i=0;_i<num_urls;_i = _i + 1)
# {
#   if(url == urls[_i])
#   {
#     return(0);
#   }
# }
 if (url_hash[url]) return (0);
 url_hash[url] = 1;

 if(debug)display("**** ADDING URL ", url, "\n");
 
 urls[num_urls] = url;
 num_urls = num_urls + 1;
 add_dir(dir_url:url);
 return(0);
}



#------------------------------------------------------#
# Extract an URL from an href= tag                     #
#------------------------------------------------------#


function extract_javascript(line, base)
{
 if(debug > 2)display("**** extract_jscript() called\n");
 
 pat = string(".*onclick.*window\\.open\\('([^',", close_parenth, "]*)'.*\\)*");

 
 url = ereg_replace(pattern:pat,
       	 	     string:line,
		     	 icase:1,
		     	 replace:"\1");
			 
			 
 if(url == line)return(0);	
 	
 while(quote >< url) url = url - quote;
 if(debug)display("**** jscript added!\n");
 add_url(url:url);			 
}


function extract_href(line, base)
{
  if(debug > 2)display("**** extract_href() called\n");

  url = ereg_replace(pattern:".*< *(a|area|frame) *.* (src|href)=([^> ]*)>.*",
       	 	     string:line,
		     	 icase:1,
		     	 replace:"\3");

  
  if(url == line)
  {
    url = ereg_replace(pattern:string("<a *.*href=", quote, "([^>", quote, " ]*)", quote),
       	 	     string:line,
		     	 icase:1,
		     	 replace:"\1");

	if (url == line) return(0);
  }
  
 

  while(quote >< url) url = url - quote;
  while(">" >< url) url = url - ">";
  
  
  _url_len = strlen(url);
  for(_url_idx=0; _url_idx<_url_len; _url_idx=_url_idx+1)
  {
	if (url[_url_idx] == " ")
	{
		url = memcpy(mem_string:url, mem_length:_url_idx);
		_url_len = strlen(url);
		_url_idx = _url_len;
	}
  }

  if(url =~ "^[a-z]+://.*"){
    	if(!(match(pattern:"http://*", string:url, icase:1)))return(0);
  }
  
  # this probably isnt correct...
  if (("mailto:" >< url) || ("http:" >< url) || ("https:" >< url) || ("javascript" >< url)) return(0);
  
  
  #
  # Suppress the dots-dots from a URL
  #
  num_dots = 0;
  orig = url;
  while("../" >< url)
  {
    url = url - "../";
    num_dots = num_dots + 1;
  }

  while("./" >< url)
  {
    url = url - "./";
  }

  base = ereg_replace(pattern:"(.*/)[^/]*$", string:base, replace:"\1");
  if(num_dots)
  {
    while(num_dots)
    {
      base = ereg_replace(pattern:"(.*/)[^/]*/$",string:base, replace:"\1");
      if(debug)display(base, "\n");
      num_dots = num_dots - 1;
    }
   url = base + url;
   if(debug)display("*** DOTS : ", orig, " -> ", url, "\n");
  }
  if(!(url[0] == "/"))
  {
    if(url !~ "[a-z]*://.*") url = base + url;
  }

  if("#" >< url)
  {
   url = ereg_replace(pattern:"(.*)#.*", 
       		      string:url,
		      replace:"\1");
  }
  add_url(url:url);
}


#------------------------------------------------------#
# Extract an URL from a FORM tag                       #
#------------------------------------------------------#
function extract_form(line, base)
{
  if(debug > 2)display("**** extract_form() called\n");

  cgi = ereg_replace(pattern:"^.*< *form .*action=([^> ]*).*>.*$",
      		     string:line,
		     icase:1,
 		     replace:"\1");

  if(cgi == line)return(0);
  while(quote >< cgi)
  {
    cgi = cgi - quote;
  }
  
  #
  # Suppress the dots-dots from a URL
  #
  num_dots = 0;
  orig = cgi;
  while("../" >< cgi)
  {
    cgi = cgi - "../";
    num_dots = num_dots + 1;
  }

  base = ereg_replace(pattern:"(.*/)[^/]*$", string:base, replace:"\1");
  if(num_dots)
  {
    while(num_dots)
    {
      base = ereg_replace(pattern:"(.*/)[^/]*/$",string:base, replace:"\1");
      if(debug)display(base, "\n");
      num_dots = num_dots - 1;
    }
   cgi = base + cgi;
   if(debug)display("*** DOTS : ", orig, " -> ", cgi, "\n");
  }
  if(!(cgi[0] == "/"))
  {
    if(cgi !~ "[a-z]*://.*") cgi = base + cgi;
  }
  
  xyz = add_cgi(cgi:cgi);
  if(debug)display("**** extract_form() returning: " , xyz, "\n");
  return(xyz);
}


#------------------------------------------------------#
# Parse a single line of the returned page             #
#------------------------------------------------------#
function parse(data, curcgi, src_page)
{

 #if(debug)display("**** parse called from src: ", src_page, "\n"); 

 # check this line for various other security issues
 pre_parse(data:data, src_page:src_page);

 # continue on to the form parser
 level = level + 1;
 if(curcgi) ___saveme  = curcgi;
 sub = ereg_replace(pattern:"[^<]*(<[^>]*>).*", replace:"\1", string:data);
		  
 if(sub == data)
 {
   level = level - 1;
   #if(debug)display("**** parse returning 0 because sub == data\n");   
   return(0);
 }

 oldr = data;		  
 while(sub)
 {
  if(sub =~ "href|frame|form|input|button|select|textarea|src")
  {


  if(sub =~ ".*(href|frame).*")
  {
   extract_href(line:sub, base:page);
  }
  
  if(match(pattern:"*onclick*", string:sub, icase:1))
  {
   extract_javascript(line:sub, base:page);
  }
  
  else
  {       
  if(sub =~ ".*[^/]form.*")
  {
  _cgi = extract_form(line:sub, base:page);
  xxx = sub;
  data = oldr - sub;
  parse(data:data, curcgi:_cgi, src_page:src_page);
  curcgi = _cgi;
  sub = xxx;
  }
  else {
   if(sub =~ ".*(input|button).*")
   {
   #
   # Extract the input name
   #
    pt = string("^.*< *(input|button) .*name=",          
                quote,
                "([^",
                quote,
                "]*)",
                quote,
               ".*>.*$");
	       
   _name = ereg_replace(pattern:pt, replace:"\2", string:sub, icase:1);
   if((_name == sub))
   {	       
   _name = ereg_replace(pattern:"^.*< *(input|button) .*name=([^<> ]*).*>.*$",
  			replace:"\2",
			string:sub,
			icase:1);
   }

    _value = "";
   
    #
    # Extract the default value
    #
    pt = string("^.*< *(input|button) .*value=",          
                quote,
                "([^",
                quote,
                "]*)",
                quote,
               ".*>.*$");
    _value = ereg_replace(pattern:pt,
  			replace:"\2",
			string:sub,
			icase:1);	
    if((!strlen(_value)) || (_value == sub))
    {
    _value = ereg_replace(pattern:"^.*< *(input|button) .*value=([^<> ]*).*>.*$",
    			replace:"\2",
			string:sub,
			icase:1);		
     if((!strlen(_value)) || (_value == sub))_value = "";			
    }					
    if(!(_name == sub))
    {
    while(quote >< _name)
    _name = _name - quote;
   
    if(!(_value == string(quote, "0", quote)))
    {
    while(quote >< _value)
    _value = _value - quote;
    }

    if(!(string(_name, " ") >< cur_args)) 
    {
    cur_args = string(cur_args, _name, " ");
    if(strlen(_value))
     {
      if(_value == string(quote, "0", quote))_value = "0";
     cur_args = string(cur_args, "[", _value, "] ");
     }
    }
  }
 }
 else
 {
 if(sub =~ ".*[^/](select|textarea)[^e].*")
  {
  _name = ereg_replace(pattern:"^.*< *(select|textarea) .*name=([^<> ]*).*>.*$",
  			replace:"\2",
			string:sub,
			icase:1);
   while(quote >< _name)
   { 
    _name = _name - quote;
   }
   if(!(string(_name, " ") >< cur_args))
    cur_args = string(cur_args, _name, " ");
  }
 else
  { 
  if(match(pattern:"*/form*", string:sub, icase:1))
  {
   if(___saveme)
   { 
    if(strlen(cur_args) > 1)
     {
      args[___saveme - 1] = cur_args;
      }
     }
     ___saveme = 0;
     cur_args = "";
     if(level > 1)
      {
       level = level - 1;
	   #if(debug)display("**** parse returning 0 because level = ", level, "\n");
       return(0);
       }
      }
     }    
    }
   }
  }
 }
  oldr = oldr - sub;
  sub = ereg_replace(pattern:"^[^<]*(<[^>]*>).*$",
       		  	  replace:"\1",
			  string:oldr,
			  icase:1);
  if(sub == oldr)sub = 0;		  
  }

  #if(debug)display("**** parse returning 0 because end of loop reached\n"); 
  return(0);
}

#------------------------------------------------------#
# Retrieve the page <page>                             #
#------------------------------------------------------#
function retr(page)
{
   if(debug)display("**** RETR ", page, "\n");
   soc = open_sock_tcp(port);
   if(soc)
   {
     str = http_get(item:page, port:port);
     if(str)
     {
     send(socket:soc, data:str); 
     code = recv_line(socket:soc, length:4096);
     r = code;
     
     #
     # Page not found
     #
     if("404" >< code)
     {
       add_404(url:page);
       if(debug)display("*** ", page, " NOT FOUND\n");
       close(soc);
       return(0);
     }
     
     #
     # Redirection (code 301)
     #
     if("301" >< code)
     {
      while(!("Location" >< r))
        r = recv_line(socket:soc, length:4096);
      
     
      location = r - string("\r\n");
      location = ereg_replace(string:location, 
      			      pattern:"Location: (.*)$",
			      replace:"\1");
		      		      
       add_url(url:location);		
       close(soc);
       return(1);	      
     }
     
     #
     # Page found (code 200)
     #
     if("200" >< code)
     {
       add_200(url:page);
       while(strlen(r) > 2)
       {
     
     #
	 # We only treat text that appear as text/{html,xml}
	 # and we skip the rest
	 #
	 r = recv_line(socket:soc, length:4096);
	 if(match(pattern:"Content-Type:*", string:r, icase:1))
     {
	   if(r !~ ".*text/(xml|html).*")
	   {
	     if(debug)display("*** SKIPPING ", page, "\n");
	     close(soc);
	     return(0);
	   }
	 }
       }

       while(r)
       {
	   # No timeout arg (yet)
           #r = recv_line(socket:soc, length:4096, timeout:2);
           r = recv_line(socket:soc, length:4096);
           level = 0;
           parse(data:r, curcgi:0, src_page:page);
       }
       close(soc);
       return(1);
     }
   }
   }
   close(soc);
   return (0);
}


#------------------------------------------------------#
# Check for various security issues in a single line   #
#------------------------------------------------------#
function pre_parse(data, src_page)
{
    # check for directory indexes
    if ("Index of /" >< data)
    {
        if(debug)display("Directory Index => ", src_page, "\n");
        
        if (add_misc_rep_list(rep:src_page))
        {
            misc_count = misc_count + 1;
            misc_report = misc_report + string("Directory index found at ", src_page, "\n");
        }
    }
    
    if ("<title>phpinfo()</title>" >< data)
    {
        if(debug)display("PHPINFO => ", src_page, "\n");
        
        if (add_misc_rep_list(rep:src_page))
        {
            misc_count = misc_count + 1;
            misc_report = misc_report + string("Extraneous phpinfo() script found at ", src_page, "\n"); 
        }       
    }
    
    php_path = ereg_replace(pattern:"(Fatal|Warning).* in <b>(.*)<\/b> on line.*", string:data, replace:"\2");
    if (!(php_path == data))
    {
        # broken #$#@#@# regexes
        php_path = php_path - string("<b>");
        if(debug)display("PHP: ", src_page, " => ", php_path, "\n");
        
        if (add_misc_rep_list(rep:src_page))
        {
            misc_count = misc_count + 1;
            misc_report = misc_report + string("PHP script discloses physical path at ", src_page, " (", php_path, ")\n");
        }
    }
    
    
    fp_save = ereg_replace(pattern:string("(.*SaveResults.*U-File=)", quote, "(.*)", quote, ".*"), string:data, replace:"\2");
    if (!(fp_save == data))
    {
        if (add_misc_rep_list(rep:src_page))
        {
            misc_count = misc_count + 1;
            misc_report = misc_report + string("FrontPage form stores results in web root at ", src_page, " (", fp_save, ")\n");
        }
        # fp_save really should be dir_from_url(src_page) + "/" + fp_save       
    }
    
    
}


#------------------------------------------------------#
# Cheesy Utility Function                              #
#------------------------------------------------------#
function memcpy(mem_string, mem_length)
{
    return(crap(length:mem_length, data:mem_string));
}

#------------------------------------------------------#
# Pull base directory from URI                         #
#------------------------------------------------------#
function dir_from_path(dir_url)
{
   _dir_copy = dir_url;
   _dir_len = strlen(dir_url);
   
   if ((_dir_len <= 1) || (dir_url == 0)) return("/");
   
   t = strstr(_dir_copy, "?");
   if(t)
   {
    _dir_copy = _dir_copy - t;
    _dir_len = strlen(_dir_copy);
   }
   
    
   for (_dir_idx=_dir_len - 1; _dir_idx > 0; _dir_idx = _dir_idx - 1)
   {
        if (_dir_idx > _dir_len) _dir_idx = 0;
        
        if ("/" >< _dir_copy[_dir_idx])
        {
            _dir_ext = memcpy(mem_string:_dir_copy, mem_length:_dir_idx); 
            return(_dir_ext);
        }
   }
   return("/");
}

#------------------------------------------------------#
# Check URL list for a match                           #
#------------------------------------------------------#
function check_url_list(url)
{
     for (VUL=0; urls[VUL]; VUL=VUL+1)
    {
        if (urls[VUL] == url)
        {
            return(1);
        }
    }
    return(0);   
}

#------------------------------------------------------#
# Add directories to the URL list                      #
#------------------------------------------------------#
function add_dir_url(url)
{
    url = string(url, "/");
    url = remove_extra_slashes(path:url);

    if (! check_url_list(url:url))
    {
        if(debug)display("**** ADDING DIR URL ", url, "\n");
        urls[num_urls] = url;
        num_urls = num_urls + 1;
    }

}

#------------------------------------------------------#
# Strip out extra /'s from a path                      #
#------------------------------------------------------#
function remove_extra_slashes(path)
{
    _path_len = strlen(path);
    _path_ret = "";
    _path_pre = "X";
    
    for(x=0; x<_path_len; x=x+1)
    {
        if (!(_path_pre == "/") || !(path[x] == "/"))
        {
            _path_ret = _path_ret + path[x];
        }
        _path_pre = path[x];
    }
    return(_path_ret);
}

#------------------------------------------------------#
# Add directories to the KB                            #
#------------------------------------------------------#
function add_dir(dir_url)
{
    if(debug > 2)display("**** ADD_DIR ", dir_url, "\n");

    _dir_len = strlen(dir_url);
    if ((_dir_len <= 1) || (dir_url == 0)) return(0);

    loop_count = 0;
	while (strlen(dir_url) > 1)
	{
		loop_count = loop_count + 1;

		dir_url = dir_from_path(dir_url:dir_url);
		if(debug > 2)display("********* CNT: " , loop_count, " => ", dir_url, "\n");

    	if (! check_url_list(url:dir_url))
    	{
        	add_dir_url(url:dir_url);
        	add_dir_list(dir:dir_url, setkb:1);
	    }
	}

	if(debug > 2)display("**** RETURNING FROM ADD_DIR\n");
    return(0);
}

#------------------------------------------------------#
# Initialize the URL list from the Directory Scanner   #
#------------------------------------------------------#

function initialize_dirs ()
{
    _dir_idx = 0;
    _dir_key = string("www/", port, "/content/directories/", _dir_idx);
    _dir_val = get_kb_item(_dir_key);
    
    while (_dir_val)
    {
        if(debug)display("**** INITIALIZE ", _dir_key, " => ", _dir_val, "\n");
    
        add_dir_list(dir:_dir_val, setkb:0);
        add_dir_url(_dir_val);
        _dir_idx = _dir_idx + 1;
        _dir_key = string("www/", port, "/content/directories/", _dir_idx);
        _dir_val = get_kb_item(_dir_key);
    }
    return(_dir_idx);
}

#
# By default, we don't retrieve more than max_pages files.
#


if(http_is_dead(port:port))exit(0);

# populate the initial url list from the KB
# save the dir count to use to add results back to the KB
dir_start = initialize_dirs();

if(debug)display("**** dir_start = ", dir_start, "\n");

# pull down the first page
retr(page:start_page);
add_vurl_list(vurl:start_page);

# keep going til we hit max_pages or run out of urls 
for(i=0;i<max_pages;i = i + 1)
{
    if (urls[i] == 0)
    {
        # if the total urls ends up being less than max we end up here...
    } else {
        if (! check_vurl_list(vurl:urls[i]))
        {
            retr(page:urls[i]);
            add_vurl_list(vurl:urls[i]);
        } else {
            if(debug > 2)display("**** ALREADY RETRIEVED ", urls[i], "\n");
        }
    }
}

#
# Analyzing time...
#

if(0)
{
 for(i=0;i<num_200;i=i+1)
 {
  if(debug)display("LINK : ", url200[i], "\n");
 }

 for(i=0;i<num_404;i=i+1)
 {
  if(debug)display("BROKEN LINK : ", url404[i], "\n");
 }
}


#-------------------------------------------------------------------#
# Just report the CGIs used at this time. We'll improve that        #
# in a newer version.                                               #
#-------------------------------------------------------------------#
my_cgis = string("\nSyntax: cginame (arguments [default value])\n");
name = string("www/",port,"/cgis");

report = "";

if (num_cgis)
{
    for(i=0;i<num_cgis;i=i+1)
    {
     if(args[i])
      {
       kb = string(cgis[i], " - ", args[i]);
       set_kb_item(name:name, value:kb);
       my_cgis = string(my_cgis, "\n", cgis[i], " ( ", args[i], ")");
      }
     else
     {
       my_cgis = string(my_cgis, "\n", cgis[i]);
     }
    }

    report = string("Web Applications: \n", my_cgis, "\n");
    
}

if (misc_count) report = report + misc_report;
if(strlen(report))
{
security_note(port:port, data:report);
}

